require 'xcodeproj'
require 'net/http'
require 'json'
require 'colored'

class APNHTTPExceptionsManager

	def self.setup_http_exceptions(installer, target_name)

		begin
##### 1. Get target
			user_target = get_user_target_with_name(installer, target_name)


##### 2. Get info.plists
	        set = Set.new
	       	user_target.resolved_build_setting("INFOPLIST_FILE").each_value do |info_plist_location|
	        	set.add?(info_plist_location)
	        end    
##### 3. Process info.plists
	        set.each do |info_plist_location|   
	     		process_info_plist(info_plist_location, user_target)        
	       	end        	

			self.print_success_message("HTTPExceptionsManager has successfully updated info.plist files for all configurations of '#{target_name}' target.")
			
		rescue Exception => e
        	print_error_message(e.message)
			Process.exit!(false)
		end


    end



    def self.get_user_target_with_name(installer, target_name)

		user_project = nil
		user_target = nil  

        installer.analysis_result.target_inspections.each_value do |target_inspection|

            user_project = Xcodeproj::Project.open(target_inspection.project_path)                      
            user_project.targets.each do |target|
            	if target.name == target_name
            		user_target = target
            		break
            	end
           	end
			unless user_target.nil?
				break  
			end
        end

        if user_target.nil?
        	raise "Unable to run HTTPExceptionsManager. Target '#{target_name}' is not found in workspace!"
        end   

        return user_target

    end


	@@http_exceptions_list = [	
								"3g.appmessages.com",
								"bannerwall.herewetest.com", 
								"3g-appmessages.s3.appcnt.com", 
								"d1ll1isslm6ii8.cloudfront.net", 
								"bannerwall.s3.appcnt.com", 
								"d8jpncozatx8a.cloudfront.net",
								"appcontpss.herewetest.com"
							]


    def self.process_info_plist(info_plist_location, user_target)  	
		
		info_plist = Xcodeproj::PlistHelper.read(info_plist_location)	
		ats_configuration = info_plist["NSAppTransportSecurity"] 
		if ats_configuration.nil?
			ats_configuration = Hash.new
		end
		exception_domains = ats_configuration["NSExceptionDomains"]
		if exception_domains.nil?
			exception_domains = Hash.new
		end

		
		@@http_exceptions_list.each do |domain|
			exception_domains[domain] = configuration_for_domain
		end


		ats_configuration["NSExceptionDomains"] = exception_domains
		info_plist["NSAppTransportSecurity"] = ats_configuration 
		Xcodeproj::PlistHelper.write(info_plist, info_plist_location)

    end 	



    def self.configuration_for_domain()
    	return  { 
    				"NSIncludesSubdomains" => true,
           			"NSExceptionRequiresForwardSecrecy" => false,
           			"NSExceptionAllowsInsecureHTTPLoads" => true
        		}
    end



   	def self.print_success_message(success_message)
   		puts ('[!] ' + success_message).cyan
   	end



   	def self.print_error_message(error_message)
		STDERR.puts ('[!] ' + error_message).red  		
   	end


end